@@ -30,6 +30,15 @@
 
 #include "cheese-camera-device.h"
 
+/**
+ * SECTION:cheese-camera-device
+ * @short_description: Object to represent a video capture device
+ * @stability: Unstable
+ * @include: cheese/cheese-camera-device.h
+ *
+ * #CheeseCameraDevice provides an abstraction of a video capture device.
+ */
+
 static void     cheese_camera_device_initable_iface_init (GInitableIface *iface);
 static gboolean cheese_camera_device_initable_init (GInitable    *initable,
                                                     GCancellable *cancellable,
@@ -44,6 +53,18 @@ G_DEFINE_TYPE_WITH_CODE (CheeseCameraDevice, cheese_camera_device, G_TYPE_OBJECT
 
 #define CHEESE_CAMERA_DEVICE_ERROR cheese_camera_device_error_quark ()
 
+/*
+ * CheeseCameraDeviceError:
+ * @CHEESE_CAMERA_DEVICE_ERROR_UNKNOWN: unknown error
+ * @CHEESE_CAMERA_DEVICE_ERROR_NOT_SUPPORTED: cancellation of device
+ * initialisation was requested, but is not supported
+ * @CHEESE_CAMERA_DEVICE_ERROR_UNSUPPORTED_CAPS: unsupported GStreamer
+ * capabilities
+ * @CHEESE_CAMERA_DEVICE_ERROR_FAILED_INITIALIZATION: the device failed to
+ * initialize for capability probing
+ *
+ * Errors that can occur during device initialization.
+ */
 enum CheeseCameraDeviceError
 {
   CHEESE_CAMERA_DEVICE_ERROR_UNKNOWN,
@@ -62,29 +83,39 @@ static gchar *supported_formats[] = {
 };
 
 /* FIXME: make this configurable */
-#define CHEESE_MAXIMUM_RATE 30 /* fps */
+/*
+ * CHEESE_MAXIMUM_RATE:
+ *
+ * The maximum framerate, in frames per second.
+ */
+static const guint CHEESE_MAXIMUM_RATE = 30;
 
 enum
 {
   PROP_0,
   PROP_NAME,
-  PROP_FILE,
-  PROP_ID,
-  PROP_API
+  PROP_DEVICE_NODE,
+  PROP_UUID,
+  PROP_V4LAPI_VERSION,
+  PROP_LAST
 };
 
-typedef struct
+static GParamSpec *properties[PROP_LAST];
+
+struct _CheeseCameraDevicePrivate
 {
-  gchar *device;
-  gchar *id;
+  gchar *device_node;
+  gchar *uuid;
   const gchar *src;
   gchar *name;
-  gint   api;
+  guint  v4lapi_version;
   GstCaps *caps;
   GList   *formats;
 
   GError *construct_error;
-} CheeseCameraDevicePrivate;
+};
+
+GQuark cheese_camera_device_error_quark (void);
 
 GQuark
 cheese_camera_device_error_quark (void)
@@ -132,15 +163,24 @@ compare_formats (gconstpointer a, gconstpointer b)
   return (d->width * d->height - c->width * c->height);
 }
 
-static
-GstCaps *
-cheese_webcam_device_filter_caps (CheeseCameraDevice *device, const GstCaps *caps, GStrv formats)
+/*
+ * cheese_camera_device_filter_caps:
+ * @device: the #CheeseCameraDevice
+ * @caps: the #GstCaps that the device supports
+ * @formats: an array of strings of video formats, in the form axb, where a and
+ * b are in units of pixels
+ *
+ * Filter the supplied @caps with %CHEESE_MAXIMUM_RATE to only allow @formats
+ * which can reach the desired framerate.
+ *
+ * Returns: the filtered #GstCaps
+ */
+static GstCaps *
+cheese_camera_device_filter_caps (CheeseCameraDevice *device, const GstCaps *caps, GStrv formats)
 {
-/*  CheeseCameraDevicePrivate *priv =
- *  CHEESE_CAMERA_DEVICE_GET_PRIVATE (device); */
   GstCaps *filter;
   GstCaps *allowed;
-  gint     i;
+  guint    i;
 
   filter = gst_caps_new_simple (formats[0],
                                 "framerate", GST_TYPE_FRACTION_RANGE,
@@ -167,11 +207,17 @@ cheese_webcam_device_filter_caps (CheeseCameraDevice *device, const GstCaps *cap
   return allowed;
 }
 
+/*
+ * cheese_camera_device_add_format:
+ * @device: a #CheeseCameraDevice
+ * @format: the #CheeseVideoFormat to add
+ *
+ * Add the supplied @format to the list of formats supported by the @device.
+ */
 static void
 cheese_camera_device_add_format (CheeseCameraDevice *device, CheeseVideoFormat *format)
 {
-  CheeseCameraDevicePrivate *priv =
-    CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
+  CheeseCameraDevicePrivate *priv =  device->priv;
   GList *l;
 
   for (l = priv->formats; l != NULL; l = l->next)
@@ -187,28 +233,48 @@ cheese_camera_device_add_format (CheeseCameraDevice *device, CheeseVideoFormat *
   priv->formats = g_list_append (priv->formats, format);
 }
 
+/*
+ * free_format_list_foreach:
+ * @data: the #CheeseVideoFormat to free
+ * @user_data: unused
+ *
+ * Free the individual #CheeseVideoFormat.
+ */
+static void
+free_format_list_foreach (gpointer data, G_GNUC_UNUSED gpointer user_data)
+{
+  g_slice_free (CheeseVideoFormat, data);
+}
+
+/*
+ * free_format_list:
+ * @device: a #CheeseCameraDevice
+ *
+ * Free the list of video formats for the @device.
+ */
 static void
 free_format_list (CheeseCameraDevice *device)
 {
-  CheeseCameraDevicePrivate *priv =
-    CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
+  CheeseCameraDevicePrivate *priv = device->priv;
 
-  GList *l;
-
-  for (l = priv->formats; l != NULL; l = l->next)
-    g_free (l->data);
-  g_list_free (priv->formats);
+  g_list_foreach (priv->formats, free_format_list_foreach, NULL);
   priv->formats = NULL;
 }
 
+/*
+ * cheese_camera_device_update_format_table:
+ * @device: a #CheeseCameraDevice
+ *
+ * Clear the current list of video formats supported by the @device and create
+ * it anew.
+ */
 static void
-cheese_webcam_device_update_format_table (CheeseCameraDevice *device)
+cheese_camera_device_update_format_table (CheeseCameraDevice *device)
 {
-  CheeseCameraDevicePrivate *priv =
-    CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
+  CheeseCameraDevicePrivate *priv = device->priv;
 
-  int i;
-  int num_structures;
+  guint i;
+  guint num_structures;
 
   free_format_list (device);
 
@@ -224,7 +290,7 @@ cheese_webcam_device_update_format_table (CheeseCameraDevice *device)
 
     if (G_VALUE_HOLDS_INT (width))
     {
-      CheeseVideoFormat *format = g_new0 (CheeseVideoFormat, 1);
+      CheeseVideoFormat *format = g_slice_new0 (CheeseVideoFormat);
 
       gst_structure_get_int (structure, "width", &(format->width));
       gst_structure_get_int (structure, "height", &(format->height));
@@ -232,8 +298,8 @@ cheese_webcam_device_update_format_table (CheeseCameraDevice *device)
     }
     else if (GST_VALUE_HOLDS_INT_RANGE (width))
     {
-      int min_width, max_width, min_height, max_height;
-      int cur_width, cur_height;
+      gint min_width, max_width, min_height, max_height;
+      gint cur_width, cur_height;
 
       min_width  = gst_value_get_int_range_min (width);
       max_width  = gst_value_get_int_range_max (width);
@@ -247,7 +313,7 @@ cheese_webcam_device_update_format_table (CheeseCameraDevice *device)
        * we use <= here (and not below) to make this work */
       while (cur_width <= max_width && cur_height <= max_height)
       {
-        CheeseVideoFormat *format = g_new0 (CheeseVideoFormat, 1);
+        CheeseVideoFormat *format = g_slice_new0 (CheeseVideoFormat);
 
         format->width  = cur_width;
         format->height = cur_height;
@@ -262,7 +328,7 @@ cheese_webcam_device_update_format_table (CheeseCameraDevice *device)
       cur_height = max_height;
       while (cur_width > min_width && cur_height > min_height)
       {
-        CheeseVideoFormat *format = g_new0 (CheeseVideoFormat, 1);
+        CheeseVideoFormat *format = g_slice_new0 (CheeseVideoFormat);
 
         format->width  = cur_width;
         format->height = cur_height;
@@ -280,11 +346,16 @@ cheese_webcam_device_update_format_table (CheeseCameraDevice *device)
   }
 }
 
+/*
+ * cheese_camera_device_get_caps:
+ * @device: a #CheeseCameraDevice
+ *
+ * Probe the #GstCaps that the @device supports.
+ */
 static void
 cheese_camera_device_get_caps (CheeseCameraDevice *device)
 {
-  CheeseCameraDevicePrivate *priv =
-    CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
+  CheeseCameraDevicePrivate *priv = device->priv;
 
   gchar               *pipeline_desc;
   GstElement          *pipeline;
@@ -294,7 +365,7 @@ cheese_camera_device_get_caps (CheeseCameraDevice *device)
   GError              *err = NULL;
 
   pipeline_desc = g_strdup_printf ("%s name=source device=%s ! fakesink",
-                                   priv->src, priv->device);
+                                   priv->src, priv->device_node);
   pipeline = gst_parse_launch (pipeline_desc, &err);
   if ((pipeline != NULL) && (err == NULL))
   {
@@ -315,12 +386,12 @@ cheese_camera_device_get_caps (CheeseCameraDevice *device)
 
       src = gst_bin_get_by_name (GST_BIN (pipeline), "source");
 
-      GST_LOG ("Device: %s (%s)\n", priv->name, priv->device);
+      GST_LOG ("Device: %s (%s)\n", priv->name, priv->device_node);
       pad        = gst_element_get_pad (src, "src");
       caps       = gst_pad_get_caps (pad);
-      priv->caps = cheese_webcam_device_filter_caps (device, caps, supported_formats);
+      priv->caps = cheese_camera_device_filter_caps (device, caps, supported_formats);
       if (!gst_caps_is_empty (priv->caps))
-        cheese_webcam_device_update_format_table (device);
+        cheese_camera_device_update_format_table (device);
       else
       {
         g_set_error_literal (&priv->construct_error,
@@ -356,7 +427,7 @@ cheese_camera_device_get_caps (CheeseCameraDevice *device)
                      CHEESE_CAMERA_DEVICE_ERROR,
                      CHEESE_CAMERA_DEVICE_ERROR_FAILED_INITIALIZATION,
                      _("Failed to initialize device %s for capability probing"),
-                     priv->device);
+                     priv->device_node);
       }
     }
     gst_element_set_state (pipeline, GST_STATE_NULL);
@@ -373,10 +444,9 @@ static void
 cheese_camera_device_constructed (GObject *object)
 {
   CheeseCameraDevice        *device = CHEESE_CAMERA_DEVICE (object);
-  CheeseCameraDevicePrivate *priv   =
-    CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
+  CheeseCameraDevicePrivate *priv   = device->priv;
 
-  priv->src = (priv->api == 2) ? "v4l2src" : "v4lsrc";
+  priv->src = (priv->v4lapi_version == 2) ? "v4l2src" : "v4lsrc";
 
   cheese_camera_device_get_caps (device);
 
@@ -388,22 +458,21 @@ static void
 cheese_camera_device_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
   CheeseCameraDevice        *device = CHEESE_CAMERA_DEVICE (object);
-  CheeseCameraDevicePrivate *priv   =
-    CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
+  CheeseCameraDevicePrivate *priv   = device->priv;
 
   switch (prop_id)
   {
     case PROP_NAME:
       g_value_set_string (value, priv->name);
       break;
-    case PROP_FILE:
-      g_value_set_string (value, priv->device);
+    case PROP_DEVICE_NODE:
+      g_value_set_string (value, priv->device_node);
       break;
-    case PROP_ID:
-      g_value_set_string (value, priv->id);
+    case PROP_UUID:
+      g_value_set_string (value, priv->uuid);
       break;
-    case PROP_API:
-      g_value_set_int (value, priv->api);
+    case PROP_V4LAPI_VERSION:
+      g_value_set_uint (value, priv->v4lapi_version);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -415,8 +484,7 @@ static void
 cheese_camera_device_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
   CheeseCameraDevice        *device = CHEESE_CAMERA_DEVICE (object);
-  CheeseCameraDevicePrivate *priv   =
-    CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
+  CheeseCameraDevicePrivate *priv   = device->priv;
 
   switch (prop_id)
   {
@@ -425,18 +493,18 @@ cheese_camera_device_set_property (GObject *object, guint prop_id, const GValue
         g_free (priv->name);
       priv->name = g_value_dup_string (value);
       break;
-    case PROP_ID:
-      if (priv->id)
-        g_free (priv->id);
-      priv->id = g_value_dup_string (value);
+    case PROP_UUID:
+      if (priv->uuid)
+        g_free (priv->uuid);
+      priv->uuid = g_value_dup_string (value);
       break;
-    case PROP_FILE:
-      if (priv->device)
-        g_free (priv->device);
-      priv->device = g_value_dup_string (value);
+    case PROP_DEVICE_NODE:
+      if (priv->device_node)
+        g_free (priv->device_node);
+      priv->device_node = g_value_dup_string (value);
       break;
-    case PROP_API:
-      priv->api = g_value_get_int (value);
+    case PROP_V4LAPI_VERSION:
+      priv->v4lapi_version = g_value_get_uint (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -448,11 +516,10 @@ static void
 cheese_camera_device_finalize (GObject *object)
 {
   CheeseCameraDevice        *device = CHEESE_CAMERA_DEVICE (object);
-  CheeseCameraDevicePrivate *priv   =
-    CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
+  CheeseCameraDevicePrivate *priv   = device->priv;
 
-  g_free (priv->device);
-  g_free (priv->id);
+  g_free (priv->device_node);
+  g_free (priv->uuid);
   g_free (priv->name);
 
   gst_caps_unref (priv->caps);
@@ -476,25 +543,53 @@ cheese_camera_device_class_init (CheeseCameraDeviceClass *klass)
   object_class->set_property = cheese_camera_device_set_property;
   object_class->constructed  = cheese_camera_device_constructed;
 
-  g_object_class_install_property (object_class, PROP_NAME,
-                                   g_param_spec_string ("name",
-                                                        NULL, NULL, NULL,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+  /**
+   * CheeseCameraDevice:name:
+   *
+   * Human-readable name of the video capture device, for display to the user.
+   */
+  properties[PROP_NAME] = g_param_spec_string ("name",
+                                               "Name of the device",
+                                               "Human-readable name of the video capture device",
+                                               NULL,
+                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
 
-  g_object_class_install_property (object_class, PROP_FILE,
-                                   g_param_spec_string ("device-file",
-                                                        NULL, NULL, NULL,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+  /**
+   * CheeseCameraDevice:device-node:
+   *
+   * Path to the device node of the video capture device.
+   */
+  properties[PROP_DEVICE_NODE] = g_param_spec_string ("device-node",
+                                                      "Device node",
+                                                      "Path to the device node of the video capture device",
+                                                      NULL,
+                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
 
-  g_object_class_install_property (object_class, PROP_ID,
-                                   g_param_spec_string ("device-id",
-                                                        NULL, NULL, NULL,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+  /**
+   * CheeseCameraDevice:uuid:
+   *
+   * UUID of the video capture device.
+   */
+  properties[PROP_UUID] = g_param_spec_string ("uuid",
+                                               "Device UUID",
+                                               "UUID of the video capture device",
+                                               NULL,
+                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+
+  /**
+   * CheeseCameraDevice:v4l-api-version:
+   *
+   * Version of the Video4Linux API that the device supports. Currently, either
+   * 1 or 2 are supported.
+   */
+  properties[PROP_V4LAPI_VERSION] = g_param_spec_uint ("v4l-api-version",
+                                                       "Video4Linux API version",
+                                                       "Version of the Video4Linux API that the device supports",
+                                                       1, 2, 2,
+                                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+
+  g_object_class_install_properties (object_class, PROP_LAST, properties);
 
-  g_object_class_install_property (object_class, PROP_API,
-                                   g_param_spec_int ("api", NULL, NULL,
-                                                     1, 2, 2,
-                                                     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
   g_type_class_add_private (klass, sizeof (CheeseCameraDevicePrivate));
 }
 
@@ -507,14 +602,13 @@ cheese_camera_device_initable_iface_init (GInitableIface *iface)
 static void
 cheese_camera_device_init (CheeseCameraDevice *device)
 {
-  CheeseCameraDevicePrivate *priv =
-    CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
+  CheeseCameraDevicePrivate *priv = device->priv = CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
 
-  priv->device = NULL;
-  priv->id     = NULL;
-  priv->src    = NULL;
-  priv->name   = g_strdup (_("Unknown device"));
-  priv->caps   = gst_caps_new_empty ();
+  priv->device_node = NULL;
+  priv->uuid = NULL;
+  priv->src = NULL;
+  priv->name = g_strdup (_("Unknown device"));
+  priv->caps = gst_caps_new_empty ();
 
   priv->formats = NULL;
 
@@ -527,8 +621,7 @@ cheese_camera_device_initable_init (GInitable    *initable,
                                     GError      **error)
 {
   CheeseCameraDevice        *device = CHEESE_CAMERA_DEVICE (initable);
-  CheeseCameraDevicePrivate *priv   =
-    CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
+  CheeseCameraDevicePrivate *priv   = device->priv;
 
   g_return_val_if_fail (CHEESE_IS_CAMERA_DEVICE (initable), FALSE);
 
@@ -551,110 +644,139 @@ cheese_camera_device_initable_init (GInitable    *initable,
   return TRUE;
 }
 
+/* public methods */
+
+/**
+ * cheese_camera_device_new:
+ * @uuid: UUID of the device, as supplied by udev
+ * @device_node: (type filename): path to the device node of the video capture
+ * device
+ * @name: human-readable name of the device, as supplied by udev
+ * @v4l_api_version: version of the Video4Linux API that the device uses. Currently
+ * either 1 or 2
+ * @error: a location to store errors
+ *
+ * Tries to create a new #CheeseCameraDevice with the supplied parameters. If
+ * construction fails, %NULL is returned, and @error is set.
+ *
+ * Returns: a new #CheeseCameraDevice, or %NULL
+ */
 CheeseCameraDevice *
-cheese_camera_device_new (const gchar *device_id,
-                          const gchar *device_file,
-                          const gchar *product_name,
-                          gint         api_version,
+cheese_camera_device_new (const gchar *uuid,
+                          const gchar *device_node,
+                          const gchar *name,
+                          guint        v4l_api_version,
                           GError     **error)
 {
   return CHEESE_CAMERA_DEVICE (g_initable_new (CHEESE_TYPE_CAMERA_DEVICE,
                                                NULL, error,
-                                               "device-id", device_id,
-                                               "device-file", device_file,
-                                               "name", product_name,
-                                               "api", api_version,
+                                               "uuid", uuid,
+                                               "device-node", device_node,
+                                               "name", name,
+                                               "v4l-api-version", v4l_api_version,
                                                NULL));
 }
 
-/* public methods */
-
 /**
  * cheese_camera_device_get_format_list:
  * @device: a #CheeseCameraDevice
  *
- * Returns: (element-type Cheese.VideoFormat) (transfer container): List of #CheeseVideoFormat
+ * Get the sorted list of #CheeseVideoFormat that the @device supports.
+ *
+ * Returns: (element-type Cheese.VideoFormat) (transfer container): list of
+ * #CheeseVideoFormat
  */
-
 GList *
 cheese_camera_device_get_format_list (CheeseCameraDevice *device)
 {
-  CheeseCameraDevicePrivate *priv =
-    CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
+  g_return_val_if_fail (CHEESE_IS_CAMERA_DEVICE (device), NULL);
 
-  return g_list_sort (g_list_copy (priv->formats), compare_formats);
+  return g_list_sort (g_list_copy (device->priv->formats), compare_formats);
 }
 
 /**
  * cheese_camera_device_get_name:
  * @device: a #CheeseCameraDevice
  *
- * Returns: (transfer none)
+ * Get a human-readable name for the device, as reported by udev, which is
+ * suitable for display to a user.
+ *
+ * Returns: (transfer none): the human-readable name of the video capture device
  */
-
 const gchar *
 cheese_camera_device_get_name (CheeseCameraDevice *device)
 {
-  CheeseCameraDevicePrivate *priv =
-    CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
+  g_return_val_if_fail (CHEESE_IS_CAMERA_DEVICE (device), NULL);
 
-  return priv->name;
+  return device->priv->name;
 }
 
-/** cheese_camera_device_get_id:
+/**
+ * cheese_camera_device_get_uuid:
  * @device: a #CheeseCameraDevice
  *
- * Returns: (transfer none)
+ * Get the UUID of the @device, as reported by udev.
+ *
+ * Returns: (transfer none): the UUID of the video capture device
  */
-
 const gchar *
-cheese_camera_device_get_id (CheeseCameraDevice *device)
+cheese_camera_device_get_uuid (CheeseCameraDevice *device)
 {
-  CheeseCameraDevicePrivate *priv =
-    CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
+  g_return_val_if_fail (CHEESE_IS_CAMERA_DEVICE (device), NULL);
 
-  return priv->id;
+  return device->priv->uuid;
 }
 
-/** cheese_camera_device_get_src:
+/**
+ * cheese_camera_device_get_src:
  * @device: a #CheeseCameraDevice
  *
- * Returns: (transfer none)
+ * Get the name of the source GStreamer element for the @device. Currently,
+ * this will be either v4lsrc or v4l2src, depending on the version of the
+ * Video4Linux API that the device supports.
+ *
+ * Returns: (transfer none): the name of the source GStreamer element
  */
-
 const gchar *
 cheese_camera_device_get_src (CheeseCameraDevice *device)
 {
-  CheeseCameraDevicePrivate *priv =
-    CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
+  g_return_val_if_fail (CHEESE_IS_CAMERA_DEVICE (device), NULL);
 
-  return priv->src;
+  return device->priv->src;
 }
 
-/** cheese_camera_device_get_device_file:
+/**
+ * cheese_camera_device_get_device_node:
  * @device: a #CheeseCameraDevice
  *
- * Returns: (transfer none)
+ * Get the path to the device node associated with the @device.
+ *
+ * Returns: (transfer none): the path to the device node of the video capture
+ * device
  */
-
 const gchar *
-cheese_camera_device_get_device_file (CheeseCameraDevice *device)
+cheese_camera_device_get_device_node (CheeseCameraDevice *device)
 {
-  CheeseCameraDevicePrivate *priv =
-    CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
+  g_return_val_if_fail (CHEESE_IS_CAMERA_DEVICE (device), NULL);
 
-  return priv->device;
+  return device->priv->device_node;
 }
 
-/** cheese_camera_device_get_device_file:
+/**
+ * cheese_camera_device_get_best_format:
  * @device: a #CheeseCameraDevice
  *
- * Returns: (transfer full): a #CheeseVideoFormat
+ * Get the #CheeseVideoFormat with the highest rsolution for this @device.
+ *
+ * Returns: (transfer full): the highest-resolution supported
+ * #CheeseVideoFormat
  */
 
 CheeseVideoFormat *
 cheese_camera_device_get_best_format (CheeseCameraDevice *device)
 {
+  g_return_val_if_fail (CHEESE_IS_CAMERA_DEVICE (device), NULL);
+
   CheeseVideoFormat *format = g_boxed_copy (CHEESE_TYPE_VIDEO_FORMAT,
                                             cheese_camera_device_get_format_list (device)->data);
 
@@ -662,21 +784,24 @@ cheese_camera_device_get_best_format (CheeseCameraDevice *device)
   return format;
 }
 
-/** cheese_camera_device_get_device_file:
+/**
+ * cheese_camera_device_get_caps_for_format:
  * @device: a #CheeseCameraDevice
+ * @format: a #CheeseVideoFormat
  *
- * Returns: (transfer full)
+ * Get the #GstCaps for the given @format on the @device.
+ *
+ * Returns: (transfer full): the #GstCaps for the given @format
  */
-
 GstCaps *
 cheese_camera_device_get_caps_for_format (CheeseCameraDevice *device,
                                           CheeseVideoFormat  *format)
 {
-  CheeseCameraDevicePrivate *priv =
-    CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
   GstCaps *desired_caps;
   GstCaps *subset_caps;
-  gint     i;
+  guint    i, length;
+
+  g_return_val_if_fail (CHEESE_IS_CAMERA_DEVICE (device), NULL);
 
   GST_INFO ("Getting caps for %dx%d", format->width, format->height);
 
@@ -687,7 +812,8 @@ cheese_camera_device_get_caps_for_format (CheeseCameraDevice *device,
                                       format->height,
                                       NULL);
 
-  for (i = 1; i < g_strv_length (supported_formats); i++)
+  length = g_strv_length (supported_formats);
+  for (i = 1; i < length; i++)
   {
     gst_caps_append (desired_caps,
                      gst_caps_new_simple (supported_formats[i],
@@ -698,7 +824,7 @@ cheese_camera_device_get_caps_for_format (CheeseCameraDevice *device,
                                           NULL));
   }
 
-  subset_caps = gst_caps_intersect (desired_caps, priv->caps);
+  subset_caps = gst_caps_intersect (desired_caps, device->priv->caps);
   gst_caps_unref (desired_caps);
 
   GST_INFO ("Got %" GST_PTR_FORMAT, subset_caps);
